import "Turbine.UI.Lotro"
import "MinstrelBuffs.MinstrelBuffWindow.TimerControl";
import "MinstrelBuffs.MinstrelBuffWindow.DragBar";

BuffWindow = class(Turbine.UI.Window);

function BuffWindow:Constructor(settings)
	Turbine.UI.Window.Constructor(self);
	self.settings = settings;
end

function BuffWindow:GetName()
	return "BuffWindow";
end

function BuffWindow:IsRunning()
	return ( self.window ~= nil );
end

function BuffWindow:ProcessKeyDown(sender, args)
    if (args.Action == 0x1000007B) then
        -- lock / unlock the control for movement
		self:UiLocked(not self.locked);
    elseif (args.Action == 0x100000B3) then
        -- hide / show ui elements
        self:UiHidden(not self.hidden);
	end
end

function BuffWindow:UiLocked(locked)
    self.locked = locked;
    if (not self.locked) then
        self.dragBar = MinstrelBuffs.MinstrelBuffWindow.DragBar(self.window, "Minstrelbuffs");
        self.dragBar:SetBarOnTop(true);
        self.dragBar:SetBarVisible(true);
        self.dragBar:SetDraggable(true);
        self.SizeChanged = function(sender, args)
            self.dragBar:Layout();
        end		
    else
        if (self.dragBar ~= nil) then
            self.dragBar:SetDraggable(false);
            self.dragBar:SetTarget(nil);
            self.dragBar = nil;
        end
        self.SizeChanged = nil;
		self.settings:SetMainPos(self.window:GetPosition());
    end
end
 
function BuffWindow:UiHidden(hide)
	self.hidden = hide;
	if (self.window ~= nil) then
		self.window:SetVisible(not hide);
	end
end

function BuffWindow:Start(posx, posy)
	self.locked = true;
	self.hidden = false;

	-- Don't create multiple windows.
	if ( self.window ~= nil ) then
		return;
	end

	-- background-Window
	--self.window = Turbine.UI.Lotro.Window();
	self.window = Turbine.UI.Window();
	if posx == nil then
		posx = 100;
	end
	if posy == nil then
		posy = 100;
	end
	self.window:SetPosition( posx, posy );
	self.window:SetSize( 300, 115 );
	self.window:SetText( "Minstrel" );
	self.window:SetVisible( true );
	self.window:SetWantsKeyEvents(true);
	self.window.KeyDown	= function(sender, args) self:ProcessKeyDown(sender, args); end
	self.window:SetBackground("MinstrelBuffs/MinstrelBuffWindow/Resources/minstrel_buffbar_Background.tga");
		
	self.window.Closed = function( sender, args )
		self:Stop();
	end

	-- Ballad effect controls
	self.balladEffectDisplays = { };
	for i = 1, 3, 1 do
		local posx = 27 + (i - 1) * 39;
		self.balladEffectDisplays[i] = Turbine.UI.Lotro.EffectDisplay();
		self.balladEffectDisplays[i]:SetParent(self.window);
		self.balladEffectDisplays[i]:SetPosition( posx, 6 );
		self.balladEffectDisplays[i]:SetVisible(false);
		self.balladEffectDisplays[i]:SetSize(28, 28);
	end
	
	-- Anthem effect controls
	self.anthemEffectDisplays = { };
	for i = 1, 6, 1 do
		local posx = 28 + (i - 1) * 40;
		self.anthemEffectDisplays[i] = Turbine.UI.Lotro.EffectDisplay()
		self.anthemEffectDisplays[i]:SetParent(self.window);
		self.anthemEffectDisplays[i]:SetPosition( posx, 45 );
		self.anthemEffectDisplays[i]:SetVisible(false);
	end
	
	-- Anthem time controls
	self.anthemTimeDisplays = { };
	for i = 1, 6, 1 do
		local posx = 29 + (i - 1) * 40;
		self.anthemTimeDisplays[i] = MinstrelBuffs.MinstrelBuffWindow.TimerControl()
		self.anthemTimeDisplays[i]:SetParent(self.window);
		self.anthemTimeDisplays[i]:SetPosition( posx, 86 );
		self.anthemTimeDisplays[i]:SetVisible(false);
	end	
end

function BuffWindow:GetPosition()
	return self.window:GetPosition();
end

function BuffWindow:Stop()
	-- Free the window which will free its child controls.
	self.window = nil;
end

function BuffWindow:SetText(text)
	--self.label:SetText(text);
end

function BuffWindow:GetEffectDisplay(index)
	return self.balladEffectDisplays[index];
end

function BuffWindow:GetAnthemEffectDisplay(index)
	return self.anthemEffectDisplays[index];
end

function BuffWindow:GetAnthemTimeDisplay(index)
	return self.anthemTimeDisplays[index];
end

function BuffWindow:ShowEffect(effect, index)
	local wnd = self:GetEffectDisplay(index);
	wnd:SetEffect(effect);
	wnd:SetVisible(true);
end

function BuffWindow:HideEffect(index)
	local wnd = self:GetEffectDisplay(index);
	wnd:SetVisible(false);
end

function BuffWindow:ShowAnthem(effect, index)
	local wnd = self:GetAnthemEffectDisplay(index);
	wnd:SetEffect(effect);
	wnd:SetVisible(true);
	
	local timeWnd = self:GetAnthemTimeDisplay(index);
	timeWnd:SetDuration(effect:GetDuration());
	timeWnd:SetStartTime(effect:GetStartTime());
	timeWnd:SetVisible(true);
end

function BuffWindow:HideAnthem(index)
	local wnd = self:GetAnthemEffectDisplay(index);
	wnd:SetVisible(false);
	local timeWnd = self:GetAnthemTimeDisplay(index);
	timeWnd:SetVisible(false);
end