import "Turbine";
import "Turbine.Gameplay";
import "MinstrelBuffs.MinstrelBuffWindow.BuffWindow";
import "MinstrelBuffs.MinstrelBuffWindow.MinstrelEffects";
import "MinstrelBuffs.MinstrelBuffWindow.MinstrelBuffCommand";
import "MinstrelBuffs.MinstrelBuffWindow.Settings";

MinstrelBuffMain = class();

function MinstrelBuffMain:Constructor()
	self.settings = MinstrelBuffs.MinstrelBuffWindow.Settings();
	self.BuffWindow = MinstrelBuffs.MinstrelBuffWindow.BuffWindow(self.settings);
end

function MinstrelBuffMain:WriteBuffsToChat()
	local placeCount = self:GetBuffCount();
	local effectCount = EffectList:GetCount();
	Turbine.Shell.WriteLine( "Current buffs: " .. placeCount );
	for	i = 1, effectCount, 1 do
		local effect = EffectList:Get(i);
		Turbine.Shell.WriteLine( effect:GetName() );		
	end
	--Turbine.Shell.WriteLine(type (Turbine.Gameplay.LocalPlayer));
	--Turbine.Shell.WriteLine( table.maxn (Turbine.Gameplay.LocalPlayer));
end

function MinstrelBuffMain:ShowWindow()
	if (EffectList.EffectAdded == nil) then
		EffectList.EffectAdded = function(sender, args) self:RefreshEffectDisplay(); end
	end	
	if (EffectList.EffectRemoved == nil) then
		EffectList.EffectRemoved = function(sender, args) self:RefreshEffectDisplay(); end
	end	
	if (EffectList.EffectCleared == nil) then
		EffectList.EffectCleared = function(sender, args) self:RefreshEffectDisplay(); end
	end
	
	if (self.BuffWindow.PositionChanged == nil) then
		self.BuffWindow.PositionChanged = function(sender, args) self:SaveMainPositionSettings(); end
	end

	local posx = self.settings:GetMainPosX();
	local posy = self.settings:GetMainPosY();
	self.BuffWindow:Start(posx, posy);
	self:RefreshEffectDisplay();	
end

function MinstrelBuffMain:HideWindow()
	if (EffectList.EffectAdded ~= nil) then
		EffectList.EffectAdded = nil;
	end	
	if (EffectList.EffectRemoved ~= nil) then
		EffectList.EffectRemoved = nil;
	end	
	if (EffectList.EffectCleared ~= nil) then
		EffectList.EffectCleared = nil;
	end
	
	if (self.BuffWindow.PositionChanged ~= nil) then
		self.BuffWindow.PositionChanged = nil;
	end
	self.BuffWindow:Stop();
end

function MinstrelBuffMain:UiLocked(locked)
	self:SaveMainPositionSettings();
	self.BuffWindow:UiLocked(locked);
end

function MinstrelBuffMain:SaveMainPositionSettings()
	--Turbine.Shell.WriteLine("SavePosition");
	if (self.BuffWindow ~= nil) then
		local posx, posy = self.BuffWindow:GetPosition();
		self.settings:SetMainPos(posx, posy);		
	end
end

function MinstrelBuffMain:SetLanguage(language)
	self.settings:SetEffectLocalizationNames(language);
	Turbine.Shell.WriteLine( "Buffnames set to langugage: " .. language );
end

function MinstrelBuffMain:GetLocalization()
	return self.settings:GetEffectLocalizationNames();
end

function MinstrelBuffMain:RefreshEffectDisplay()
	local effectCount = EffectList:GetCount();
	local balladEffectPlace = 1;
	local anthemEffectPlace = 1;
	local sortedEffects = {};
	for	i = 1, effectCount, 1 do
		sortedEffects[i] = EffectList:Get(i);
	end
	table.sort(sortedEffects, function(a,b) return a:GetStartTime() < b:GetStartTime() end);
	
	for	i = 1, effectCount, 1 do
		--local effect = EffectList:Get(i);
		local effect = sortedEffects[i];
		if (balladEffectPlace <= 3 and self:IsValidBalladEffect(effect:GetName())) then
			self.BuffWindow:ShowEffect(effect, balladEffectPlace);
			balladEffectPlace = balladEffectPlace + 1;
		end
		if (anthemEffectPlace <= 6 and self:IsValidHymnEffect(effect:GetName())) then
			self.BuffWindow:ShowAnthem(effect, anthemEffectPlace)
			anthemEffectPlace = anthemEffectPlace + 1;
		end
	end
	
	if (balladEffectPlace <= 3) then
		for i = balladEffectPlace, 3, 1 do
			self.BuffWindow:HideEffect(balladEffectPlace);
		end
	end
	if (anthemEffectPlace <= 6) then
		for i = anthemEffectPlace, 6, 1 do
			self.BuffWindow:HideAnthem(anthemEffectPlace);
		end
	end
end

function MinstrelBuffMain:IsValidBalladEffect(effectName)
	if (self:GetLocalization() == "en") then
		return ValidBalladEffectsEn[effectName] ~= nil;
	else
		return ValidBalladEffectsDe[effectName] ~= nil;
	end
end

function MinstrelBuffMain:IsValidHymnEffect(effectName)
	if (self:GetLocalization() == "en") then
		return ValidHymnEffectsEn[effectName] ~= nil;
	else
		return ValidHymnEffectsDe[effectName] ~= nil;
	end
end

function MinstrelBuffMain:GetBuffCount()
	local count = 0;
	count = EffectList:GetCount();
	return count;
end

mbMain = MinstrelBuffMain();
--mbCommand = MinstrelBuffs.MinstrelBuffWindow.MinstrelBuffCommand(mbMain);
MinstrelBuffCommand.MinstrelBuffMain = mbMain;
Turbine.Shell.AddCommand( "minstrelbuff", MinstrelBuffCommand);
Turbine.Shell.AddCommand( "mb", MinstrelBuffCommand);

LocalPlayer = Turbine.Gameplay.LocalPlayer.GetInstance();
EffectList = LocalPlayer:GetEffects();
mbMain:ShowWindow();
MinstrelBuffCommand:ShowHelpText();
